@@ -46,6 +46,18 @@ sub namespaces {
     return( @{ $self->{ namespaces } } );
 }
 
+=head2 namespaces_byprefix()
+
+Returns the namespace URI associated to a given metadataPrefix.
+
+=cut
+
+sub namespaces_byprefix {
+    my ($self, $prefix) = @_;
+    return $self->{namespaces_byprefix}->{$prefix};
+}
+
+
 =head2 schemas()
 
 =cut
@@ -55,32 +67,56 @@ sub schemas {
     return( @{ $self->{ schemas } } );
 }
 
-## SAX Handlers
+=head2 schemas_byprefix()
+
+Returns the schema URI associated to a given metadataPrefix.
+
+=cut
+
+sub schemas_byprefix {
+    my ($self, $prefix) = @_;
+    return $self->{schemas_byprefix}->{$prefix};
+}
+
+my $xmlns_oai = "http://www.openarchives.org/OAI/2.0/";
+
+# SAX Handlers
 
 sub start_element {
     my ( $self, $element ) = @_;
-    if ( $element->{ Name } eq 'ListMetadataFormats' ) { 
+    return $self->SUPER::start_element($element) unless $element->{NamespaceURI} eq $xmlns_oai;  # should be error?
+
+    if ( $element->{ LocalName } eq 'ListMetadataFormats' ) { 
 	$self->{ insideList } = 1;
     } else {
 	$self->SUPER::start_element( $element );
     }
-    push( @{ $self->{ tagStack } }, $element->{ Name } );
+    push( @{ $self->{ tagStack } }, $element->{ LocalName } );
 }
 
 sub end_element {
     my ( $self, $element ) = @_;
-    my $name = $element->{ Name };
+    return $self->SUPER::end_element($element) unless $element->{NamespaceURI} eq $xmlns_oai;  # should be error?
+
+    my $name = $element->{ LocalName };
     if ( $name eq 'ListMetadataFormats' ) {
 	$self->{ insideList } = 0;
+    } elsif ( $name eq 'metadataFormat' ) {
+        my $nspf = delete $self->{ _currpf };
+        $self->{ namespaces_byprefix }->{ $nspf } = delete $self->{ _currns };
+        $self->{ schemas_byprefix }->{ $nspf } = delete $self->{ _currxs };
     } elsif ( $name eq 'metadataPrefix' ) {
 	push( @{ $self->{ metadataPrefixes } }, $self->{ metadataPrefix } );
+        $self->{ _currpf } = $self->{ metadataPrefix };
 	$self->{ metadataPrefix } = '';
     } elsif ( $name eq 'schema' ) {
         push( @{ $self->{ schemas } }, $self->{schema } );
+        $self->{ _currxs } = $self->{ schema };
         $self->{ schema } = '';
     } elsif ( $name eq 'metadataNamespace' ) {
         push( @{ $self->{ namespaces } }, $self->{ metadataNamespace } );
-        $self->{ namespace } = '';
+        $self->{ _currns } = $self->{ metadataNamespace };
+        $self->{ metadataNamespace } = '';
     } else {
 	$self->SUPER::end_element( $element );
     }
